import os
from setuptools import setup, find_packages

path = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(path, "README.md"), "r") as f:
    readme = f.read()

setup(
    name="articledata",
    version="0.1.2",
    description="Utility library for trading article data.",
    long_description=readme,
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    keywords="articles media news journalism",
    author="Friedrich Lindenberg, Heleen Emanuel",
    author_email="friedrich@pudo.org",
    url="http://github.com/pudo/articledata",
    license="MIT",
    packages=find_packages(exclude=["ez_setup", "examples", "test"]),
    namespace_packages=[],
    package_data={"": ["articledata/py.typed"]},
    include_package_data=True,
    zip_safe=False,
    test_suite="nose.collector",
    install_requires=["languagecodes", "pydantic"],
    extras_require={
        "dev": ["mypy", "wheel", "twine", "pytest", "types-PyYAML"],
    },
    tests_require=[],
    entry_points={},
)



from hashlib import sha1
from functools import cached_property
from typing import Any, List, Optional, Set, Tuple, Union, Generator, Callable
from urllib.parse import urlencode, urljoin, urlparse, parse_qsl, ParseResult


class URL(object):
    def __init__(self, url: str) -> None:
        if isinstance(url, URL):
            url = url.url
        self.url = url

    @cached_property
    def parsed(self) -> ParseResult:
        return urlparse(self.url)

    @cached_property
    def domain(self) -> str:
        domain = self.parsed.hostname or "localhost"
        return domain.strip(".").lower()

    @cached_property
    def scheme(self) -> str:
        if self.parsed.scheme is None or not len(self.parsed.scheme):
            return "http"
        return self.parsed.scheme.lower().strip()

    def clean(self, query_ignore: Set[str]) -> "URL":
        parsed = self.parsed._replace(fragment="")
        parsed = parsed._replace(netloc=parsed.netloc.lower())
        cleaned_query: List[Tuple[str, str]] = []
        for key, value in parse_qsl(parsed.query):
            if key not in query_ignore:
                cleaned_query.append((key, value))
        parsed = parsed._replace(query=urlencode(cleaned_query))
        return URL(parsed.geturl())

    @cached_property
    def id(self) -> str:
        parsed = self.parsed._replace(fragment="")
        parsed = parsed._replace(netloc=parsed.netloc.lower())
        parsed = parsed._replace(path=parsed.path.rstrip("/"))
        parsed = parsed._replace(scheme="http")
        norm = parsed.geturl().encode("utf-8")
        return sha1(norm).hexdigest()

    def join(self, text: str) -> Optional["URL"]:
        try:
            return URL(urljoin(self.url, text))
        except (TypeError, ValueError):
            return None

    @classmethod
    def __get_validators__(
        cls,
    ) -> Generator[Callable[[Any], Union[str, "URL"]], None, None]:
        yield cls.validate

    @classmethod
    def validate(cls, text: Any) -> Union[str, "URL"]:
        if not isinstance(text, (str, URL)) or text is None:
            raise TypeError("URL is not a string: %r", type(text))
        return text

    def __str__(self) -> str:
        return self.url

    def __repr__(self) -> str:
        return self.url

    def __hash__(self) -> int:
        return hash(self.id)

    def __eq__(self, other: Any) -> bool:
        if isinstance(other, URL):
            return other.id == self.id
        return str(other) == self.url



from typing import List, Optional
from pydantic import BaseModel, Field

from articledata.url import URL

# cf.:
# * https://github.com/adbar/trafilatura/blob/master/trafilatura/metadata.py
# * https://schema.org/Article


class Article(BaseModel):
    id: str
    title: str
    section: Optional[str]
    date: Optional[str]
    url: URL
    tags: List[str] = []
    site: str  # e.g. "occrp"
    language: str = Field(min_length=3, max_length=3)  # 3-letter or 2-letter?
    locale: str = Field(min_length=2, max_length=10)
    bylines: List[str]
    description: Optional[str]
    lede: Optional[str]
    text: str
    extracted_at: str
